/* attention: there is something wrong; explain: what design mistakes were made? what would you do instead? */

#include "buggy-is-prime.h"

/*
 * this function takes an int64_t argument k.
 * it returns a pointer to an int64_t value that is the first number >= k that is a prime number.
 */
int64_t* allocate_next_prime_number(int64_t k)
{
   /*
    * loop goes on until a prime number is found
    * 
    * note that k is passed by value, so we can increment it without affecting the calling function
    */
   while(true)
   {
      int64_t* n = new int64_t(k);
      if(is_prime(n)) return n;
      k++;
   }
}

/*
 * receives an int64_t pointer (int64_t* n)
 * checks whether *n, the int64_t value stored at that address, is a prime number
 */
bool is_prime(int64_t* n)
{
   /*
    * explicitly handle multiples of two and three
    */
   if((*n%2 == 0) || (*n%3 == 0))
   {
      /*
       * deallocate the pointer since the value is no longer needed
       */
      delete n;
      return false;
   }
   
   /*
    * now we only need to check divisibility by 6k-1 and 6k+1, gaining us a factor 3 speedup
    */
   for(int64_t i = 5; *n >= i*i; i += 6)
   {
      if((*n % i == 0) || (*n % (i+2) == 0))
      {
         /*
          * deallocate the pointer since the value is no longer needed
          */
         delete n;
         return false;
      }
   }

   // hooray, it's a prime number
   // pointer may still be needed, so we don't delete it and simply return "true"
   //
   return true;
}

